@@ -3,6 +3,8 @@
 Test suite for vdirsyncer.
 '''
 
+import hypothesis.strategies as st
+
 from vdirsyncer.utils.vobject import normalize_item
 
 
@@ -17,7 +19,7 @@ def assert_item_equals(a, b):
 VCARD_TEMPLATE = u'''BEGIN:VCARD
 VERSION:3.0
 FN:Cyrus Daboo
-N:Daboo;Cyrus
+N:Daboo;Cyrus;;;
 ADR;TYPE=POSTAL:;2822 Email HQ;Suite 2821;RFCVille;PA;15213;USA
 EMAIL;TYPE=PREF:cyrus@example.com
 NICKNAME:me
@@ -25,7 +27,7 @@ NOTE:Example VCard.
 ORG:Self Employed
 TEL;TYPE=VOICE:412 605 0499
 TEL;TYPE=FAX:412 605 0705
-URL:http://www.example.com
+URL;VALUE=URI:http://www.example.com
 X-SOMETHING:{r}
 UID:{uid}
 END:VCARD'''
@@ -66,3 +68,17 @@ UID:{uid}
 X-SOMETHING:{r}
 HAHA:YES
 END:FOO'''
+
+printable_characters_strategy = st.text(
+    st.characters(blacklist_categories=(
+        'Cc', 'Cs'
+    ))
+)
+
+uid_strategy = st.text(
+    st.characters(blacklist_categories=(
+        'Zs', 'Zl', 'Zp',
+        'Cc', 'Cs'
+    )),
+    min_size=1
+).filter(lambda x: x.strip() == x)
