@@ -3,9 +3,10 @@ from textwrap import dedent
 
 import pytest
 
-import vdirsyncer.cli.config  # noqa
-import vdirsyncer.cli.utils  # noqa
 from vdirsyncer import cli, exceptions
+from vdirsyncer.cli.config import Config, _parse_config_value
+
+import vdirsyncer.cli.utils  # noqa
 
 
 invalid = object()
@@ -17,7 +18,7 @@ def read_config(tmpdir, monkeypatch):
         errors = []
         monkeypatch.setattr('vdirsyncer.cli.cli_logger.error', errors.append)
         f = io.StringIO(dedent(cfg.format(base=str(tmpdir))))
-        rv = vdirsyncer.cli.config.read_config(f)
+        rv = Config.from_fileobject(f)
         monkeypatch.undo()
         return errors, rv
     return inner
@@ -27,7 +28,7 @@ def read_config(tmpdir, monkeypatch):
 def parse_config_value(capsys):
     def inner(s):
         try:
-            rv = vdirsyncer.cli.config.parse_config_value(s)
+            rv = _parse_config_value(s)
         except ValueError:
             return invalid
         else:
@@ -37,45 +38,39 @@ def parse_config_value(capsys):
 
 
 def test_read_config(read_config):
-    errors, (general, pairs, storages) = read_config(u'''
+    errors, c = read_config(u'''
         [general]
         status_path = /tmp/status/
 
         [pair bob]
-        a = bob_a
-        b = bob_b
-        foo = bar
-        bam = true
+        a = "bob_a"
+        b = "bob_b"
         collections = null
 
         [storage bob_a]
-        type = filesystem
-        path = /tmp/contacts/
-        fileext = .vcf
+        type = "filesystem"
+        path = "/tmp/contacts/"
+        fileext = ".vcf"
         yesno = false
         number = 42
 
         [storage bob_b]
-        type = carddav
-
-        [bogus]
-        lol = true
+        type = "carddav"
         ''')
 
-    assert general == {'status_path': '/tmp/status/'}
-    assert pairs == {'bob': ('bob_a', 'bob_b',
-                             {'collections': None, 'bam': True, 'foo': 'bar'})}
-    assert storages == {
+    assert c.general == {'status_path': '/tmp/status/'}
+
+    assert set(c.pairs) == {'bob'}
+    bob = c.pairs['bob']
+    assert bob.collections is None
+
+    assert c.storages == {
         'bob_a': {'type': 'filesystem', 'path': '/tmp/contacts/', 'fileext':
                   '.vcf', 'yesno': False, 'number': 42,
                   'instance_name': 'bob_a'},
         'bob_b': {'type': 'carddav', 'instance_name': 'bob_b'}
     }
 
-    assert len(errors) == 1
-    assert errors[0].startswith('Unknown section')
-    assert 'bogus' in errors[0]
-
 
 def test_missing_collections_param(read_config):
     with pytest.raises(exceptions.UserError) as excinfo:
@@ -84,19 +79,32 @@ def test_missing_collections_param(read_config):
             status_path = /tmp/status/
 
             [pair bob]
-            a = bob_a
-            b = bob_b
+            a = "bob_a"
+            b = "bob_b"
 
             [storage bob_a]
-            type = lmao
+            type = "lmao"
 
             [storage bob_b]
-            type = lmao
+            type = "lmao"
             ''')
 
     assert 'collections parameter missing' in str(excinfo.value)
 
 
+def test_invalid_section_type(read_config):
+    with pytest.raises(exceptions.UserError) as excinfo:
+        read_config(u'''
+            [general]
+            status_path = /tmp/status/
+
+            [bogus]
+        ''')
+
+    assert 'Unknown section' in str(excinfo.value)
+    assert 'bogus' in str(excinfo.value)
+
+
 def test_storage_instance_from_config(monkeypatch):
     def lol(**kw):
         assert kw == {'foo': 'bar', 'baz': 1}
@@ -112,19 +120,19 @@ def test_missing_general_section(read_config):
     with pytest.raises(exceptions.UserError) as excinfo:
         read_config(u'''
             [pair my_pair]
-            a = my_a
-            b = my_b
+            a = "my_a"
+            b = "my_b"
             collections = null
 
             [storage my_a]
-            type = filesystem
-            path = {base}/path_a/
-            fileext = .txt
+            type = "filesystem"
+            path = "{base}/path_a/"
+            fileext = ".txt"
 
             [storage my_b]
-            type = filesystem
-            path = {base}/path_b/
-            fileext = .txt
+            type = "filesystem"
+            path = "{base}/path_b/"
+            fileext = ".txt"
             ''')
 
     assert 'Invalid general section.' in str(excinfo.value)
@@ -163,33 +171,58 @@ def test_invalid_collections_arg(read_config):
         status_path = /tmp/status/
 
         [pair foobar]
-        a = foo
-        b = bar
+        a = "foo"
+        b = "bar"
         collections = [null]
 
         [storage foo]
-        type = filesystem
-        path = /tmp/foo/
-        fileext = .txt
+        type = "filesystem"
+        path = "/tmp/foo/"
+        fileext = ".txt"
 
         [storage bar]
-        type = filesystem
-        path = /tmp/bar/
-        fileext = .txt
+        type = "filesystem"
+        path = "/tmp/bar/"
+        fileext = ".txt"
         ''')
 
     assert 'Expected string' in str(excinfo.value)
 
 
+def test_duplicate_sections(read_config):
+    with pytest.raises(exceptions.UserError) as excinfo:
+        read_config(u'''
+        [general]
+        status_path = /tmp/status/
+
+        [pair foobar]
+        a = "foobar"
+        b = "bar"
+        collections = null
+
+        [storage foobar]
+        type = "filesystem"
+        path = "/tmp/foo/"
+        fileext = ".txt"
+
+        [storage bar]
+        type = "filesystem"
+        path = "/tmp/bar/"
+        fileext = ".txt"
+        ''')
+
+    assert 'Name "foobar" already used' in str(excinfo.value)
+
+
 def test_parse_config_value(parse_config_value):
     x = parse_config_value
 
     assert x('123  # comment!') is invalid
 
-    assert x('True') == ('True', 1)
-    assert x('False') == ('False', 1)
-    assert x('Yes') == ('Yes', 1)
-    assert x('None') == ('None', 1)
+    assert x('True') is invalid
+    assert x('False') is invalid
+    assert x('Yes') is invalid
+    assert x('None') is invalid
     assert x('"True"') == ('True', 0)
     assert x('"False"') == ('False', 0)
 
@@ -198,14 +231,12 @@ def test_parse_config_value(parse_config_value):
     assert x('false') == (False, 0)
     assert x('null') == (None, 0)
     assert x('3.14') == (3.14, 0)
-    assert x('') == ('', 0)
+    assert x('') == ('', 1)
     assert x('""') == ('', 0)
 
 
-def test_validate_pair_section_collections_param():
-    def x(val):
-        return cli.config._validate_pair_section({'collections': val})
-
+def test_validate_collections_param():
+    x = cli.config._validate_collections_param
     x(None)
     x(["c", "a", "b"])
     pytest.raises(ValueError, x, [None])
