@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
 import json
+import sys
 import unicodedata
 from textwrap import dedent
 
@@ -9,25 +10,23 @@ from hypothesis import example, given
 
 import pytest
 
-from vdirsyncer.utils.compat import PY2, to_native, to_unicode
-
 
 def test_simple_run(tmpdir, runner):
     runner.write_with_general(dedent('''
     [pair my_pair]
-    a = my_a
-    b = my_b
+    a = "my_a"
+    b = "my_b"
     collections = null
 
     [storage my_a]
-    type = filesystem
-    path = {0}/path_a/
-    fileext = .txt
+    type = "filesystem"
+    path = "{0}/path_a/"
+    fileext = ".txt"
 
     [storage my_b]
-    type = filesystem
-    path = {0}/path_b/
-    fileext = .txt
+    type = "filesystem"
+    path = "{0}/path_b/"
+    fileext = ".txt"
     ''').format(str(tmpdir)))
 
     tmpdir.mkdir('path_a')
@@ -56,19 +55,19 @@ def test_sync_inexistant_pair(tmpdir, runner):
 def test_debug_connections(tmpdir, runner):
     runner.write_with_general(dedent('''
     [pair my_pair]
-    a = my_a
-    b = my_b
+    a = "my_a"
+    b = "my_b"
     collections = null
 
     [storage my_a]
-    type = filesystem
-    path = {0}/path_a/
-    fileext = .txt
+    type = "filesystem"
+    path = "{0}/path_a/"
+    fileext = ".txt"
 
     [storage my_b]
-    type = filesystem
-    path = {0}/path_b/
-    fileext = .txt
+    type = "filesystem"
+    path = "{0}/path_b/"
+    fileext = ".txt"
     ''').format(str(tmpdir)))
 
     tmpdir.mkdir('path_a')
@@ -87,19 +86,19 @@ def test_debug_connections(tmpdir, runner):
 def test_empty_storage(tmpdir, runner):
     runner.write_with_general(dedent('''
     [pair my_pair]
-    a = my_a
-    b = my_b
+    a = "my_a"
+    b = "my_b"
     collections = null
 
     [storage my_a]
-    type = filesystem
-    path = {0}/path_a/
-    fileext = .txt
+    type = "filesystem"
+    path = "{0}/path_a/"
+    fileext = ".txt"
 
     [storage my_b]
-    type = filesystem
-    path = {0}/path_b/
-    fileext = .txt
+    type = "filesystem"
+    path = "{0}/path_b/"
+    fileext = ".txt"
     ''').format(str(tmpdir)))
 
     tmpdir.mkdir('path_a')
@@ -139,18 +138,18 @@ def test_collections_cache_invalidation(tmpdir, runner):
 
     runner.write_with_general(dedent('''
     [storage foo]
-    type = filesystem
-    path = {0}/foo/
-    fileext = .txt
+    type = "filesystem"
+    path = "{0}/foo/"
+    fileext = ".txt"
 
     [storage bar]
-    type = filesystem
-    path = {0}/bar/
-    fileext = .txt
+    type = "filesystem"
+    path = "{0}/bar/"
+    fileext = ".txt"
 
     [pair foobar]
-    a = foo
-    b = bar
+    a = "foo"
+    b = "bar"
     collections = ["a", "b", "c"]
     ''').format(str(tmpdir)))
 
@@ -169,18 +168,18 @@ def test_collections_cache_invalidation(tmpdir, runner):
 
     runner.write_with_general(dedent('''
     [storage foo]
-    type = filesystem
-    path = {0}/foo/
-    fileext = .txt
+    type = "filesystem"
+    path = "{0}/foo/"
+    fileext = ".txt"
 
     [storage bar]
-    type = filesystem
-    path = {0}/bar2/
-    fileext = .txt
+    type = "filesystem"
+    path = "{0}/bar2/"
+    fileext = ".txt"
 
     [pair foobar]
-    a = foo
-    b = bar
+    a = "foo"
+    b = "bar"
     collections = ["a", "b", "c"]
     ''').format(str(tmpdir)))
 
@@ -209,18 +208,18 @@ def test_collections_cache_invalidation(tmpdir, runner):
 def test_invalid_pairs_as_cli_arg(tmpdir, runner):
     runner.write_with_general(dedent('''
     [storage foo]
-    type = filesystem
-    path = {0}/foo/
-    fileext = .txt
+    type = "filesystem"
+    path = "{0}/foo/"
+    fileext = ".txt"
 
     [storage bar]
-    type = filesystem
-    path = {0}/bar/
-    fileext = .txt
+    type = "filesystem"
+    path = "{0}/bar/"
+    fileext = ".txt"
 
     [pair foobar]
-    a = foo
-    b = bar
+    a = "foo"
+    b = "bar"
     collections = ["a", "b", "c"]
     ''').format(str(tmpdir)))
 
@@ -242,17 +241,17 @@ def test_multiple_pairs(tmpdir, runner):
         for name_a, name_b in ('foo', 'bar'), ('bam', 'baz'):
             yield dedent('''
             [pair {a}{b}]
-            a = {a}
-            b = {b}
+            a = "{a}"
+            b = "{b}"
             collections = null
             ''').format(a=name_a, b=name_b)
 
             for name in name_a, name_b:
                 yield dedent('''
                 [storage {name}]
-                type = filesystem
-                path = {path}
-                fileext = .txt
+                type = "filesystem"
+                path = "{path}"
+                fileext = ".txt"
                 ''').format(name=name, path=str(tmpdir.mkdir(name)))
 
     runner.write_with_general(''.join(get_cfg()))
@@ -277,7 +276,6 @@ def test_multiple_pairs(tmpdir, runner):
         st.characters(
             blacklist_characters=set(
                 u'./\x00'  # Invalid chars on POSIX filesystems
-                + (u';' if PY2 else u'')  # https://bugs.python.org/issue16374
             ),
             # Surrogates can't be encoded to utf-8 in Python
             blacklist_categories=set(['Cs'])
@@ -288,33 +286,33 @@ def test_multiple_pairs(tmpdir, runner):
     min_size=1
 ))
 @example(collections=[u'persönlich'])
+@example(collections=set('aA'))
 def test_create_collections(subtest, collections):
-    collections = set(to_native(x, 'utf-8') for x in collections)
 
     @subtest
     def test_inner(tmpdir, runner):
         runner.write_with_general(dedent('''
         [pair foobar]
-        a = foo
-        b = bar
+        a = "foo"
+        b = "bar"
         collections = {colls}
 
         [storage foo]
-        type = filesystem
-        path = {base}/foo/
-        fileext = .txt
+        type = "filesystem"
+        path = "{base}/foo/"
+        fileext = ".txt"
 
         [storage bar]
-        type = filesystem
-        path = {base}/bar/
-        fileext = .txt
+        type = "filesystem"
+        path = "{base}/bar/"
+        fileext = ".txt"
         '''.format(base=str(tmpdir), colls=json.dumps(list(collections)))))
 
         result = runner.invoke(
             ['discover'],
             input='y\n' * 2 * (len(collections) + 1)
         )
-        assert not result.exception
+        assert not result.exception, result.output
 
         # Macs normally operate on the HFS+ file system which normalizes paths.
         # That is, if you save a file with accented é in it (u'\xe9') for
@@ -324,10 +322,12 @@ def test_create_collections(subtest, collections):
         #
         # Quoted from
         # https://stackoverflow.com/questions/18137554/how-to-convert-path-to-mac-os-x-path-the-almost-nfd-normal-form  # noqa
-        u = lambda xs: set(
-            unicodedata.normalize('NFKD', to_unicode(x, 'utf-8'))
-            for x in xs
-        )
+        def u(xs):
+            xs = (unicodedata.normalize('NFKD', x) for x in xs)
+            if sys.platform == 'darwin':
+                xs = (x.lower() for x in xs)
+            return set(xs)
+
         assert u(x.basename for x in tmpdir.join('foo').listdir()) == \
             u(x.basename for x in tmpdir.join('bar').listdir()) == \
             u(collections)
@@ -335,25 +335,25 @@ def test_create_collections(subtest, collections):
         result = runner.invoke(
             ['sync'] + ['foobar/' + x for x in collections]
         )
-        assert not result.exception
+        assert not result.exception, result.output
 
 
 def test_ident_conflict(tmpdir, runner):
     runner.write_with_general(dedent('''
     [pair foobar]
-    a = foo
-    b = bar
+    a = "foo"
+    b = "bar"
     collections = null
 
     [storage foo]
-    type = filesystem
-    path = {base}/foo/
-    fileext = .txt
+    type = "filesystem"
+    path = "{base}/foo/"
+    fileext = ".txt"
 
     [storage bar]
-    type = filesystem
-    path = {base}/bar/
-    fileext = .txt
+    type = "filesystem"
+    path = "{base}/bar/"
+    fileext = ".txt"
     '''.format(base=str(tmpdir))))
 
     foo = tmpdir.mkdir('foo')
@@ -384,14 +384,14 @@ def test_ident_conflict(tmpdir, runner):
 def test_unknown_storage(tmpdir, runner, existing, missing):
     runner.write_with_general(dedent('''
     [pair foobar]
-    a = foo
-    b = bar
+    a = "foo"
+    b = "bar"
     collections = null
 
     [storage {existing}]
-    type = filesystem
-    path = {base}/{existing}/
-    fileext = .txt
+    type = "filesystem"
+    path = "{base}/{existing}/"
+    fileext = ".txt"
     '''.format(base=str(tmpdir), existing=existing)))
 
     tmpdir.mkdir(existing)
@@ -413,3 +413,164 @@ def test_no_configured_pairs(tmpdir, runner, cmd):
     result = runner.invoke([cmd])
     assert result.output == 'critical: Nothing to do.\n'
     assert result.exception.code == 5
+
+
+@pytest.mark.parametrize('resolution,expect_foo,expect_bar', [
+    (['command', 'cp'], 'UID:lol\nfööcontent', 'UID:lol\nfööcontent')
+])
+def test_conflict_resolution(tmpdir, runner, resolution, expect_foo,
+                             expect_bar):
+    runner.write_with_general(dedent('''
+    [pair foobar]
+    a = "foo"
+    b = "bar"
+    collections = null
+    conflict_resolution = {val}
+
+    [storage foo]
+    type = "filesystem"
+    fileext = ".txt"
+    path = "{base}/foo"
+
+    [storage bar]
+    type = "filesystem"
+    fileext = ".txt"
+    path = "{base}/bar"
+    '''.format(base=str(tmpdir), val=json.dumps(resolution))))
+
+    foo = tmpdir.join('foo')
+    bar = tmpdir.join('bar')
+    fooitem = foo.join('lol.txt').ensure()
+    fooitem.write('UID:lol\nfööcontent')
+    baritem = bar.join('lol.txt').ensure()
+    baritem.write('UID:lol\nbööcontent')
+
+    r = runner.invoke(['discover'])
+    assert not r.exception
+
+    r = runner.invoke(['sync'])
+    assert not r.exception
+
+    assert fooitem.read() == expect_foo
+    assert baritem.read() == expect_bar
+
+
+@pytest.mark.parametrize('partial_sync', ['error', 'ignore', 'revert', None])
+def test_partial_sync(tmpdir, runner, partial_sync):
+    runner.write_with_general(dedent('''
+    [pair foobar]
+    a = "foo"
+    b = "bar"
+    collections = null
+    {partial_sync}
+
+    [storage foo]
+    type = "filesystem"
+    fileext = ".txt"
+    path = "{base}/foo"
+
+    [storage bar]
+    type = "filesystem"
+    read_only = true
+    fileext = ".txt"
+    path = "{base}/bar"
+    '''.format(
+        partial_sync=('partial_sync = {}\n'.format(partial_sync)
+                      if partial_sync else ''),
+        base=str(tmpdir)
+    )))
+
+    foo = tmpdir.mkdir('foo')
+    bar = tmpdir.mkdir('bar')
+
+    foo.join('other.txt').write('UID:other')
+    bar.join('other.txt').write('UID:other')
+
+    baritem = bar.join('lol.txt')
+    baritem.write('UID:lol')
+
+    r = runner.invoke(['discover'])
+    assert not r.exception
+
+    r = runner.invoke(['sync'])
+    assert not r.exception
+
+    fooitem = foo.join('lol.txt')
+    fooitem.remove()
+
+    r = runner.invoke(['sync'])
+
+    if partial_sync == 'error':
+        assert r.exception
+        assert 'Attempted change' in r.output
+    elif partial_sync == 'ignore':
+        assert baritem.exists()
+        r = runner.invoke(['sync'])
+        assert not r.exception
+        assert baritem.exists()
+    else:
+        assert baritem.exists()
+        r = runner.invoke(['sync'])
+        assert not r.exception
+        assert baritem.exists()
+        assert fooitem.exists()
+
+
+def test_fetch_only_necessary_params(tmpdir, runner):
+    fetched_file = tmpdir.join('fetched_flag')
+    fetch_script = tmpdir.join('fetch_script')
+    fetch_script.write(dedent('''
+    set -e
+    touch "{}"
+    echo ".txt"
+    '''.format(str(fetched_file))))
+
+    runner.write_with_general(dedent('''
+    [pair foobar]
+    a = "foo"
+    b = "bar"
+    collections = null
+
+    [pair bambar]
+    a = "bam"
+    b = "bar"
+    collections = null
+
+    [storage foo]
+    type = "filesystem"
+    path = "{path}"
+    fileext = ".txt"
+
+    [storage bar]
+    type = "filesystem"
+    path = "{path}"
+    fileext = ".txt"
+
+    [storage bam]
+    type = "filesystem"
+    path = "{path}"
+    fileext.fetch = ["command", "sh", "{script}"]
+    '''.format(path=str(tmpdir.mkdir('bogus')), script=str(fetch_script))))
+
+    def fetched():
+        try:
+            fetched_file.remove()
+            return True
+        except Exception:
+            return False
+
+    r = runner.invoke(['discover'])
+    assert not r.exception
+    assert fetched()
+
+    r = runner.invoke(['sync', 'foobar'])
+    assert not r.exception
+    assert not fetched()
+
+    r = runner.invoke(['sync'])
+    assert not r.exception
+    assert fetched()
+
+    r = runner.invoke(['sync', 'bambar'])
+    assert not r.exception
+    assert fetched()
