@@ -17,9 +17,8 @@ import click_threading
 
 from . import cli_logger
 from .. import BUGTRACKER_HOME, DOCS_HOME, exceptions
-from ..sync import IdentConflict, StorageEmpty, SyncConflict
+from ..sync import IdentConflict, PartialSync, StorageEmpty, SyncConflict
 from ..utils import expand_path, get_storage_init_args
-from ..utils.compat import to_native
 
 try:
     import Queue as queue
@@ -38,8 +37,8 @@ DISCOVERY_CACHE_VERSION = 1
 class _StorageIndex(object):
     def __init__(self):
         self._storages = dict(
-            caldav='vdirsyncer.storage.dav.CaldavStorage',
-            carddav='vdirsyncer.storage.dav.CarddavStorage',
+            caldav='vdirsyncer.storage.dav.CalDAVStorage',
+            carddav='vdirsyncer.storage.dav.CardDAVStorage',
             filesystem='vdirsyncer.storage.filesystem.FilesystemStorage',
             http='vdirsyncer.storage.http.HttpStorage',
             singlefile='vdirsyncer.storage.singlefile.SingleFileStorage',
@@ -72,7 +71,7 @@ class JobFailed(RuntimeError):
 
 
 # TODO: Making this a decorator would be nice
-def handle_cli_error(status_name=None):
+def handle_cli_error(status_name=None, e=None):
     '''
     Print a useful error message for the current exception.
 
@@ -81,7 +80,10 @@ def handle_cli_error(status_name=None):
     '''
 
     try:
-        raise
+        if e is not None:
+            raise e
+        else:
+            raise
     except exceptions.UserError as e:
         cli_logger.critical(e)
     except StorageEmpty as e:
@@ -95,6 +97,13 @@ def handle_cli_error(status_name=None):
                 status_name=status_name
             )
         )
+    except PartialSync as e:
+        cli_logger.error(
+            '{status_name}: Attempted change on {storage}, which is read-only'
+            '. Set `partial_sync` in your pair section to `ignore` to ignore '
+            'those changes, or `revert` to revert them on the other side.'
+            .format(status_name=status_name, storage=e.storage)
+        )
     except SyncConflict as e:
         cli_logger.error(
             '{status_name}: One item changed on both sides. Resolve this '
@@ -147,8 +156,7 @@ def handle_cli_error(status_name=None):
         import traceback
         tb = traceback.format_tb(tb)
         if status_name:
-            msg = 'Unknown error occured for {}'.format(
-                coerce_native(status_name))
+            msg = 'Unknown error occured for {}'.format(status_name)
         else:
             msg = 'Unknown error occured'
 
@@ -168,7 +176,7 @@ def _get_collections_cache_key(pair):
     m = hashlib.sha256()
     j = json.dumps([
         DISCOVERY_CACHE_VERSION,
-        pair.options.get('collections', None),
+        pair.collections,
         pair.config_a,
         pair.config_b,
     ], sort_keys=True)
@@ -269,9 +277,9 @@ def _discover_from_config(config):
 def _handle_collection_not_found(config, collection, e=None):
     storage_name = config.get('instance_name', None)
 
-    cli_logger.error('{}No collection {} found for storage {}.'
-                     .format('{}\n'.format(e) if e else '',
-                             json.dumps(collection), storage_name))
+    cli_logger.warning('{}No collection {} found for storage {}.'
+                       .format('{}\n'.format(e) if e else '',
+                               json.dumps(collection), storage_name))
 
     if click.confirm('Should vdirsyncer attempt to create it?'):
         storage_type = config['type']
@@ -293,7 +301,7 @@ def _handle_collection_not_found(config, collection, e=None):
 
 def _print_collections(base_config, discovered):
     instance_name = base_config['instance_name']
-    cli_logger.info('{}:'.format(coerce_native(instance_name)))
+    cli_logger.info('{}:'.format(instance_name))
     for args in discovered.values():
         collection = args['collection']
         if collection is None:
@@ -308,7 +316,7 @@ def _print_collections(base_config, discovered):
 
         cli_logger.info('  - {}{}'.format(
             json.dumps(collection),
-            ' ("{}")'.format(coerce_native(displayname))
+            ' ("{}")'.format(displayname)
             if displayname and displayname != collection
             else ''
         ))
@@ -317,7 +325,7 @@ def _print_collections(base_config, discovered):
 def _collections_for_pair_impl(status_path, pair, list_collections=False):
     handled_collections = set()
 
-    shortcuts = pair.options['collections']
+    shortcuts = pair.collections
     if shortcuts is None:
         shortcuts = [None]
 
@@ -465,7 +473,7 @@ def handle_storage_init_error(cls, config):
             u'{} storage doesn\'t take the parameters: {}'
             .format(cls.storage_name, u', '.join(invalid)))
 
-    if not problems:  # XXX: Py2: Proper reraise
+    if not problems:
         raise e
 
     raise exceptions.UserError(
@@ -585,18 +593,3 @@ def assert_permissions(path, wanted):
         cli_logger.warning('Correcting permissions of {} from {:o} to {:o}'
                            .format(path, permissions, wanted))
         os.chmod(path, wanted)
-
-
-def coerce_native(x, encoding='utf-8'):
-    # XXX: Remove with Python 3 only
-    try:
-        return str(x)
-    except UnicodeError:
-        pass
-
-    try:
-        return to_native(x, encoding=encoding)
-    except UnicodeError:
-        pass
-
-    return repr(x)
