@@ -27,6 +27,7 @@ def _generate_path_reserved_chars():
         yield x.upper()
         yield x.lower()
 
+
 _path_reserved_chars = frozenset(_generate_path_reserved_chars())
 del _generate_path_reserved_chars
 
@@ -84,6 +85,8 @@ def _parse_xml(content):
 
 
 def _merge_xml(items):
+    if not items:
+        return []
     rv = items[0]
     for item in items[1:]:
         rv.extend(item.getiterator())
@@ -598,7 +601,7 @@ class DAVStorage(Storage):
         root = _parse_xml(response.content)
 
         rv = self._parse_prop_responses(root)
-        for href, etag, prop in rv:
+        for href, etag, _prop in rv:
             yield href, etag
 
     def get_meta(self, key):
@@ -814,7 +817,7 @@ class CalDAVStorage(DAVStorage):
                                             headers=headers)
             root = _parse_xml(response.content)
             rv = self._parse_prop_responses(root)
-            for href, etag, prop in rv:
+            for href, etag, _prop in rv:
                 yield href, etag
 
 
