@@ -12,7 +12,6 @@ from atomicwrites import atomic_write
 from .base import Item, Storage
 from .. import exceptions
 from ..utils import checkfile, expand_path
-from ..utils.compat import iteritems, itervalues
 from ..utils.vobject import join_collection, split_collection
 
 logger = logging.getLogger(__name__)
@@ -54,11 +53,11 @@ class SingleFileStorage(Storage):
         collections = ["from a", "from b"]
 
         [storage my_calendar_local]
-        type = singlefile
+        type = "singlefile"
         path = ~/.calendars/%s.ics
 
         [storage my_calendar_remote]
-        type = caldav
+        type = "caldav"
         url = https://caldav.example.org/
         #username =
         #password =
@@ -70,11 +69,11 @@ class SingleFileStorage(Storage):
         b = my_calendar_remote
 
         [storage my_calendar_local]
-        type = singlefile
+        type = "singlefile"
         path = ~/my_calendar.ics
 
         [storage my_calendar_remote]
-        type = caldav
+        type = "caldav"
         url = https://caldav.example.org/username/my_calendar/
         #username =
         #password =
@@ -167,7 +166,7 @@ class SingleFileStorage(Storage):
             etag = item.hash
             self._items[item.ident] = item, etag
 
-        return ((href, etag) for href, (item, etag) in iteritems(self._items))
+        return ((href, etag) for href, (item, etag) in self._items.items())
 
     def get(self, href):
         if self._items is None or not self._at_once:
@@ -218,7 +217,7 @@ class SingleFileStorage(Storage):
                 'synchronization and make sure absolutely no other program is '
                 'writing into the same file.'.format(self.path))
         text = join_collection(
-            (item.raw for item, etag in itervalues(self._items)),
+            item.raw for item, etag in self._items.values()
         )
         try:
             with atomic_write(self.path, mode='wb', overwrite=True) as f:
@@ -233,9 +232,6 @@ class SingleFileStorage(Storage):
         self._at_once = True
         try:
             yield self
-        except:
-            raise
-        else:
             self._write()
         finally:
             self._at_once = False
