@@ -154,6 +154,7 @@ def split_collection(text):
         item.subcomponents.extend(inline)
         yield u'\r\n'.join(item.dump_lines())
 
+
 _default_join_wrappers = {
     u'VCALENDAR': u'VCALENDAR',
     u'VEVENT': u'VCALENDAR',
@@ -255,7 +256,7 @@ class _Component(object):
         stack = []
         rv = []
         try:
-            for i, line in enumerate(lines):
+            for _i, line in enumerate(lines):
                 if line.startswith(u'BEGIN:'):
                     c_name = line[len(u'BEGIN:'):].strip().upper()
                     stack.append(cls(c_name, [], []))
@@ -270,7 +271,7 @@ class _Component(object):
                         stack[-1].props.append(line)
         except IndexError:
             raise ValueError('Parsing error at line {}. Check the debug log '
-                             'for more information.'.format(i + 1))
+                             'for more information.'.format(_i + 1))
 
         if multiple:
             return rv
